//#define A52DEC_CS
#if defined (A52DEC_CS)
#include <windows.h>
#endif
#include <stdlib.h>
#include <string.h>
#include "a52dec.h"
#include "ffmpeg_stub.h"

#if ! defined (A52DEC_CS)
#define InitializeCriticalSection(cs)
#define DeleteCriticalSection(cs)
#define EnterCriticalSection(cs)
#define LeaveCriticalSection(cs)
#else
static CRITICAL_SECTION cs;
#endif

a52dec_t *a52dec_open(a52dec_t *x, ao_instance_t *(*output_open)(void))
{
  memset(x,0,sizeof *x);

  if (NULL==(x->buf=__av_malloc(3840)))
	goto error;

  x->bufptr = x->buf;
  x->bufpos = x->buf + 7;

  x->bit_rate=0;
  x->sample_rate=0;
  x->flags=0;

  x->disable_accel = 0;
  x->disable_dynrng = 1;
  x->disable_adjust = 0;
  x->gain = 1;

  if (NULL==(x->output=ao_open(output_open)))
    goto error;

  if (NULL==(x->state=__a52_init(x->disable_accel?0:MM_ACCEL_DJBFFT)))
    goto error;

  return x;
error:
  a52dec_close(x);

  return NULL;
}

a52dec_t *a52dec_close(a52dec_t *x)
{
  if (NULL!=x->state)
    __a52_free (x->state);

  if (NULL!=x->output)
    ao_close (x->output);

  if (NULL!=x->buf)
    __av_free(x->buf);

  return x;
}

void a52dec_flush(a52dec_t *x)
{
  x->bufptr = x->buf;
  x->bufpos = x->buf + 7;
}

void a52dec_decode_data (a52dec_t *x, uint8_t * start, uint8_t * end)
{
    int len;
	int ret;

    while (1) {
      len = end - start;
      if (!len)
        break;
      if (len > x->bufpos - x->bufptr)
        len = x->bufpos - x->bufptr;
      memcpy (x->bufptr, start, len);
      x->bufptr += len;
      start += len;
      if (x->bufptr == x->bufpos) {
        if (x->bufpos == x->buf + 7) {
          int length;

          EnterCriticalSection(&cs);
          length = __a52_syncinfo (x->buf, &x->flags, &x->sample_rate, &x->bit_rate);
          LeaveCriticalSection(&cs);
          if (!length) {
#if defined (ORIGINAL)
            fprintf (stderr, "skip\n");
#endif
            for (x->bufptr = x->buf; x->bufptr < x->buf + 6; x->bufptr++)
              x->bufptr[0] = x->bufptr[1];
            continue;
          }
          x->bufpos = x->buf + length;
        } else {
          sample_t level, bias;
          int i;

          EnterCriticalSection(&cs);
          ret = ao_setup (x->output, x->sample_rate, &x->flags, &level, &bias);
          LeaveCriticalSection(&cs);
          if (ret)
            goto error;
          if (!x->disable_adjust)
            x->flags |= A52_ADJUST_LEVEL;
          level *= x->gain;
          EnterCriticalSection(&cs);
          ret = __a52_frame (x->state, x->buf, &x->flags, &level, bias);
          LeaveCriticalSection(&cs);
          if (ret)
            goto error;
          if (x->disable_dynrng) {
            EnterCriticalSection(&cs);
            __a52_dynrng (x->state, NULL, NULL);
            LeaveCriticalSection(&cs);
		  }
          for (i = 0; i < 6; i++) {
            EnterCriticalSection(&cs);
            ret = __a52_block (x->state);
            LeaveCriticalSection(&cs);
            if (ret)
              goto error;
            EnterCriticalSection(&cs);
            ret = ao_play (x->output, x->flags, __a52_samples (x->state));
            LeaveCriticalSection(&cs);
            if (ret)
              goto error;
          }
          x->bufptr = x->buf;
          x->bufpos = x->buf + 7;
#if defined (ORIGINAL)
          print_fps (0);
#endif
          continue;
        error:
#if defined (ORIGINAL)
          fprintf (stderr, "error\n");
#endif
          x->bufptr = x->buf;
          x->bufpos = x->buf + 7;
        }
    }
  }
}

void a52dec_start()
{
  InitializeCriticalSection(&cs);
}

void a52dec_stop()
{
  DeleteCriticalSection(&cs);
}
